/****************************************************************************
 Module
   LauncherService.c

 Revision
   1.0.1

 Description
   This is a template file for implementing a simple service under the
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/16/12 09:58 jec      began conversion from TemplateFSM.c
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "LauncherService.h"
#include "SPIServiceLauncher.h"

#include <sys/attribs.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <xc.h>

#include "ES_DeferRecall.h"
#include "ES_Port.h"
#include "terminal.h"
#include "dbprintf.h"
#include "PIC32PortHAL.h"
//#include "PIC32_SPI_HAL.h"
#include "PIC32_SPI_HAL_KW.h"
//#include "PIC32_AD_Lib.h"
//#include "PWM_PIC32.h"


/*----------------------------- Module Defines ----------------------------*/
#define SERVO_TIMER_LENGTH 1000 // 1 sec
#define LAUNCHER_TIMER_LENGTH 1000 // 5 sec

#define SERVO_PERIOD (50000) //50Hz
#define DCMOTOR_PR (300) //assumes time constant of ~100us

#define MOTOR_PORT _Port_B
#define SERVO1_PIN _Pin_3 // Pin  7
#define SERVO2_PIN _Pin_5 // Pin 14

#define DCMOTOR_PORT _Port_B
#define DCMOTOR_PIN  _Pin_2 // Pin 6

#define LIMITSWITCH_PORT _Port_B
#define LIMITSWITCH_PIN _Pin_4 // Pin 11

#define OPEN_POSITION 1500 //1300
#define CLOSED_POSITION 3000

#define KEYSTROKE

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this service.They should be functions
   relevant to the behavior of this service
*/
void CloseServo(uint8_t);
void OpenServo(uint8_t);
void UpdateDCMotor(uint16_t DutyCycle);

void ConfigureTimer2(void);
void ConfigureTimer3(void);
void ConfigureOC1_Servo1(void);
void ConfigureOC2_Servo2(void);
void ConfigureOC4_DCMotor(void);

void StartTMR2(void);
void StartTMR3(void);

/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;

static LauncherState_t CurrentState;
static uint8_t CurrServo;
static bool ServoOpening;
static uint8_t ShootCount = 0;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitLauncherService

 Parameters
     uint8_t : the priority of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any
     other required initialization for this service
 Notes

 Author
     Lauren Taylor
****************************************************************************/
bool InitLauncherService(uint8_t Priority)
{
  ES_Event_t ThisEvent;

  MyPriority = Priority;
  
  //Configure Timer 2 for servo PWMs
  ConfigureTimer2();
  //Configure OC1 and OC2 for servo PWMs
  ConfigureOC1_Servo1();
  ConfigureOC2_Servo2();
  
  ConfigureTimer3();
  ConfigureOC4_DCMotor();
  
  //Initialize closed position
  CloseServo(1);
  CloseServo(2);
  
  printf("Initialize Done\n\r");

  //Put us into the initial Waiting state
  CurrentState = Waiting;
    
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService(MyPriority, ThisEvent) == true)
  {
    return true;
  }
  else
  {
    return false;
  }
}

/****************************************************************************
 Function
     PostLauncherService

 Parameters
     EF_Event_t ThisEvent,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     Lauren Taylor
****************************************************************************/
bool PostLauncherService(ES_Event_t ThisEvent)
{
  return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunLauncherService

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes

 Author
    Lauren Taylor
****************************************************************************/
ES_Event_t RunLauncherService(ES_Event_t ThisEvent)
{
  ES_Event_t ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
  
  switch(CurrentState)
  {
    case(Waiting):
    {
      switch(ThisEvent.EventType)
      {
        case(ES_INIT):
        {
            printf("\rES_INIT\r\n");
        }
        break;
        
        /* Prepare to open 1st servo for beginning reload */
        case(ES_RECEIVEDCOMMAND): // Received shoot command
        {
            if(ThisEvent.EventParam == 0xFF) {
                CurrentState = Waiting;
                ES_Event_t NewEvent;
                NewEvent.EventType = ES_INIT;
                PostLauncherService(NewEvent);
            } else {
                printf("\rMove Servo1 for beginning reload\r\n");
                CurrServo = 1;
                ServoOpening = 1;
                ES_Event_t NewEvent;
                CurrentState = LocalReload;
                NewEvent.EventType = ES_RELOAD;
                PostLauncherService(NewEvent);
            }
        }
        break;
        
#ifdef KEYSTROKE
        case EV_NEW_KEY:
        {
            if (ThisEvent.EventParam == '1')
            {
                printf("\rMove Servo1 for beginning reload\r\n");
                CurrServo = 1;
                ServoOpening = 1;
                ES_Event_t NewEvent;
                CurrentState = LocalReload;
                NewEvent.EventType = ES_RELOAD;
                PostLauncherService(NewEvent);
            }
        }
        break;
#endif //KEYSTROKE
        
        default:
        {}
        break;
      }
    }
    break;
    
    case(LocalReload):
    {
      switch(ThisEvent.EventType)
      {
        case(ES_RELOAD):
        {
            printf("\rServo 1 opening\r\n");
            OpenServo(CurrServo);
            ES_Timer_InitTimer(SERVO_TIMER, SERVO_TIMER_LENGTH);
        }
        break;
        
        case(ES_TIMEOUT):
        {
            ES_Event_t NewEvent;
            if(CurrServo == 1) { // Servo 1 is moving
                if(ServoOpening == 1) { // Servo 1 needs to close
                    printf("\rServo 1 closing\r\n");
                    CloseServo(CurrServo);
                    ServoOpening = 0; // Currently closing a servo
                    CurrServo = 2; // Next time operate on servo 2
                    ES_Timer_InitTimer(SERVO_TIMER, SERVO_TIMER_LENGTH); // Start servo timer again while it closes
                } else {
                    printf("\rMove to ES_SHOOT\r\n");
                    ServoOpening = 1;
                    CurrentState = Shooting; // Start shooting since reload is complete
                    NewEvent.EventType = ES_SHOOT;
                    PostLauncherService(NewEvent);
                }
            } else { // Servo 2 is moving
                if(ServoOpening == 0) { //  Servo 2 needs to open
                    printf("\rServo 2 opening\r\n");
                    ServoOpening = 1;
                    OpenServo(CurrServo);
                    ES_Timer_InitTimer(SERVO_TIMER, SERVO_TIMER_LENGTH); // Start servo timer again while it opens
                } else { // Servo 2 needs to close
                    printf("\rServo 2 closing\r\n");
                    ServoOpening = 0;
                    CloseServo(CurrServo);
                    CurrServo = 1; // Will take us to a part of code in ES_TIMEOUT where we change state to ES_SHOOT
                    ES_Timer_InitTimer(SERVO_TIMER, SERVO_TIMER_LENGTH); // Start servo timer again while it closes
                }
            }
        }
        break;
        
        case(ES_RECEIVEDCOMMAND):
        {
            if(ThisEvent.EventParam == 0xFF) {
                CurrentState = Waiting;
                ES_Event_t NewEvent;
                NewEvent.EventType = ES_INIT;
                PostLauncherService(NewEvent);
            }
        }
        break;
        
        default:
        {}
        break;
      }
    }
    break;
    
    case(Shooting):
    {
      switch(ThisEvent.EventType)
      {
        case(ES_SHOOT):
        {
          //printf("\rES_SHOOT\r\n");
          // Start shoot motor
          UpdateDCMotor(DCMOTOR_PR); //Set duty cycle to full speed
          ES_Timer_InitTimer(LAUNCHER_TIMER, LAUNCHER_TIMER_LENGTH); // Start launcher timer to pull back rack
          ShootCount++;
        }
        break;
        
//        case(ES_TIMEOUT):
//        {
//            printf("\rShoot number %d\r\n", ShootCount);
//            ES_Event_t NewEvent;
//            
//            //Stop shoot motor
//            UpdateDCMotor(0); //Set duty cycle to 0
//            
//            if(ShootCount < 3) { // Start reload again if there are still balls left
//                CurrentState = LocalReload;
//                NewEvent.EventType = ES_RELOAD;
//                PostLauncherService(NewEvent);
//                printf("\rReturn to reload\r\n");
//            } else { // Return to reload position
//                CurrentState = Waiting;
//                ShootCount = 0;
//                NewEvent.EventType = ES_SENDCOMMAND;
//                NewEvent.EventParam = 1; // Send command of 1, corresponding to "Return to reload"
//                PostSPIServiceLauncher(NewEvent);
//            }
//        }
//        break;
        case(ES_SHOOT_COMPLETE):
        {
            //printf("\rES_SHOOT_COMPLETE");
            printf("\rShoot number %d\r\n", ShootCount);
            ES_Event_t NewEvent;

            //Stop shoot motor
            UpdateDCMotor(0); //Set duty cycle to 0

            if(ShootCount < 3) { // Start reload again if there are still balls left
                CurrentState = LocalReload;
                NewEvent.EventType = ES_RELOAD;
                PostLauncherService(NewEvent);
                printf("\rReturn to reload\r\n");
            } else { // Return to reload position
                CurrentState = Waiting;
                ShootCount = 0;
                NewEvent.EventType = ES_UPDATEMESSAGE;
                NewEvent.EventParam = 2; // Send command of 1, corresponding to "Return to reload"
                PostSPIServiceLauncher(NewEvent);
            }
        }
        break;
        
        case(ES_RECEIVEDCOMMAND):
        {
            if(ThisEvent.EventParam == 0xFF) {
                CurrentState = Waiting;
                ES_Event_t NewEvent;
                NewEvent.EventType = ES_INIT;
                PostLauncherService(NewEvent);
            }
        }
        break;
        
        default:
        {}
        break;
      }
    }
    break;
    
    default:
    {}
    break;
  }  

  return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/

//void ServoSetup(uint8_t channel, WhichTimer_t timer, PWM_PinMap_t pin) {
//        PWMSetup_SetFreqOnTimer(50, timer);
//        PWMSetup_AssignChannelToTimer(channel, timer);
//        PWMSetup_MapChannelToOutputPin(channel, pin);
//        PWMOperate_SetDutyOnChannel(25,channel);}

//////////////////////////////////////////////////////////////////////////
//void CloseServo(uint8_t channel) {
//    PWMOperate_SetPulseWidthOnChannel((uint16_t)2500, channel);
//}

void CloseServo(uint8_t WhichServo)
{
    if (1 == WhichServo) {
        OC1RS = CLOSED_POSITION;
        T2CONbits.ON = 0; // Turn timer off while StartTMR3 is happening
        StartTMR2();
    } else if (2 == WhichServo) {
        OC2RS = CLOSED_POSITION;
        T2CONbits.ON = 0; // Turn timer off while StartTMR3 is happening
        StartTMR2();
    }
}

/////////////////////////////////////////////////////////////////////////
//void OpenServo(uint8_t channel) {
//    PWMOperate_SetPulseWidthOnChannel((uint16_t)0, channel);
//}

void OpenServo(uint8_t WhichServo)
{
    if (1 == WhichServo) {
        OC1RS = OPEN_POSITION;
        T2CONbits.ON = 0; // Turn timer off while StartTMR3 is happening
        StartTMR2();    
    } else if (2 == WhichServo) {
        OC2RS = OPEN_POSITION;
        T2CONbits.ON = 0; // Turn timer off while StartTMR3 is happening
        StartTMR2();
    }
}

////////////////////////////////////////////////////////////////////////
void ConfigureTimer2(void)
{
    //Disable Timer 2
    T2CONbits.ON = 0;
    //Select the internal PBCLK source
    T2CONbits.TCS = 0;
    //Clear timer gate
    T2CONbits.TGATE = 0;
    //Set prescale to 8
    T2CONbits.TCKPS = 0b011;
    //Set match count (period) to 0xFFFF
    PR2 = SERVO_PERIOD;
    StartTMR2();
}

void StartTMR2(void) {
    IFS0CLR = _IFS0_T2IF_MASK; // Clear any pending flag
    TMR2 = 0; // Kick off timer by zeroing count
    T2CONbits.ON = 1; // Enable timer
}

void ConfigureOC1_Servo1(void)
{   
    //Configure pin for output PWM
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, SERVO1_PIN); //RB3
    // Map Pin RB3 to OC1
    RPB3R = 0b0101;
    // Disable OC 
    OC1CONbits.ON = 0;
    // Select Timer2 as source
    OC1CONbits.OCTSEL = 0;
    // Set PWM mode with no fault detect
    OC1CONbits.OCM = 0b110;
    // Set initial duty cycle to 0
    OC1RS = 0;
    OC1R = 0;
    // Enable OC system
    OC1CONbits.ON = 1;
}

void ConfigureOC2_Servo2(void)
{
    //Configure pin for output PWM
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, SERVO2_PIN); //RB5
    // Map Pin RB5 to OC2
    RPB5R = 0b0101;
    // Disable OC 
    OC2CONbits.ON = 0;
    // Select Timer2 as source
    OC2CONbits.OCTSEL = 0;
    // Set PWM mode with no fault detect
    OC2CONbits.OCM = 0b110;
    // Set initial duty cycle to 0
    OC2RS = 0;
    OC2R = 0;
    // Enable OC system
    OC2CONbits.ON = 1;
}

///////////////////////////////////////////////////////////////////////////

void ConfigureTimer3(void)
{
    // Disable Timer 3
    T3CONbits.ON = 0;
    // Select internal PBCLK source
    T3CONbits.TCS = 0;
    // Clear timer gate
    T3CONbits.TGATE = 0;
    // Set prescale to 8
    T3CONbits.TCKPS = 0b011;
    // Set period
    PR3 = DCMOTOR_PR;
    StartTMR3();
}

void StartTMR3(void) {
    IFS0CLR = _IFS0_T3IF_MASK; // Clear any pending flag
    TMR3 = 0; // Kick  off timer by zeroing count
    T3CONbits.ON = 1; // Enable timer
}

void ConfigureOC4_DCMotor(void)
{
    //Configure pin for output PWM
    PortSetup_ConfigureDigitalOutputs(MOTOR_PORT, DCMOTOR_PIN); //RB2
    // Map Pin RB2 to OC4
    RPB2R = 0b0101;
    // Disable OC 
    OC4CONbits.ON = 0;
    // Select Timer3 as source
    OC4CONbits.OCTSEL = 1;
    // Set PWM mode with no fault detect
    OC4CONbits.OCM = 0b110;
    // Set initial duty cycle to 0
    OC4RS = 0;
    OC4R = 0;
    // Enable OC system
    OC4CONbits.ON = 1;
}

void UpdateDCMotor(uint16_t DutyCycle)
{
    OC4RS = DutyCycle;
//    T3CONbits.ON = 0; // Turn timer off while StartTMR3 is happening
//    StartTMR3();
}
/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/

