#include "bitdefs.h"
#include "PIC32PortHAL.h"
#include <stdint.h>
#include <xc.h>

//	Function Prototypes

_Bool PortSetup_ConfigureDigitalInputs( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);
_Bool PortSetup_ConfigureDigitalOutputs( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);
_Bool PortSetup_ConfigureAnalogInputs( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);
_Bool PortSetup_ConfigurePullUps( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);
_Bool PortSetup_ConfigurePullDowns( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);
_Bool PortSetup_ConfigureOpenDrain( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);
_Bool PortSetup_ConfigureChangeNotification( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin);

static uint16_t portA_val = 0;
static uint16_t portB_val = 1;
static uint32_t portA_DIO = BIT0HI | BIT1HI | BIT2HI | BIT3HI | BIT4HI;
static uint16_t portB_DIO = 0xffff;

static uint32_t portA_AIO = BIT0HI | BIT1HI;
static uint32_t portB_AIO = BIT0HI | BIT1HI | BIT2HI | BIT3HI | BIT4HI | BIT12HI | BIT13HI | BIT14HI | BIT15HI;

uint16_t pinCheck;
//_Bool testDigitalI;



// Functions


/****************************************************************************
 Function
    PortSetup_ConfigureDigitalOutputs

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin(s) to be configured as digital outputs

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port as digital outputs
Example
   PortSetup_ConfigureDigitalOutputs(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigureDigitalOutputs( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    //Set returnVal to False
    _Bool returnVal = 0;
    
    if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_DIO; //portA_DO;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ANSELA = ANSELA&(~WhichPin); //ANSEL LO
            TRISA = (~WhichPin)&TRISA; // TRIS LO
            returnVal = 1;        
        }
    }
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_DIO; //portB_DO;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ANSELB = ANSELB&(~WhichPin); // ANSEL LO
            TRISB = (~WhichPin)&TRISB;   // TRIS LO
            returnVal = 1;
        }        
    }
    // If pins not valid, return false
    else{
        returnVal = 0;
    }
    return returnVal;
}



/****************************************************************************
 Function
    PortSetup_ConfigureDigitalInputs

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin to be configured as digital inputs

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port as digital inputs, disabling analog input(s).
Example
   PortSetup_ConfigureDigitalInputs(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigureDigitalInputs( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    _Bool returnVal = 0;
    if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_DIO; //portA_DI;\
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ANSELA = ANSELA&(~WhichPin); // ANSEL LO
            TRISA = (WhichPin)|TRISA; // TRIS HI
            returnVal = 1;        
        }
    }
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_DIO; //portB_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ANSELB = ANSELB&(~WhichPin); // ANSEL LO
            TRISB = (WhichPin)|TRISB; // TRIS HI   
            returnVal = 1;
        }        
    }
    // If invalid pin/port, return false
    else{
        returnVal = 0;
    }
    return returnVal;
}
        

/****************************************************************************
 Function
    PortSetup_ConfigureAnalogInputs

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin(s) to be configured as analog inputs

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port as analog inputs
Example
   PortSetup_ConfigureAnalogInputs(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigureAnalogInputs( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    _Bool returnVal = 0;
    if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_AIO; //portA_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ANSELA = ANSELA|WhichPin; // ANSEL HI
            TRISA = (WhichPin)|TRISA; // TRIS HI
            returnVal = 1;        
        }
    }
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_AIO; //portB_DI;
        if(pinCheck == WhichPin){
            ANSELB = ANSELB|WhichPin; //ANSEL HI
            TRISB = (WhichPin)|TRISB; // TRIS HI   
            returnVal = 1;
        }        
    }
    // If invalid pin/port, return false
    else{
        returnVal = 0;
    }
    return returnVal;
}
    
 
/****************************************************************************
 Function
    PortSetup_ConfigurePullUps

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin(s) to be configured with weak pull-ups

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port with weak pull-ups
Example
   PortSetup_ConfigurePullUps(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigurePullUps( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    _Bool returnVal = 0;
    if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_DIO; //portA_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            CNPUA = CNPUA | WhichPin; //Configure pin as pull-up (set high)
            returnVal = 1;        
        }
    }
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_DIO; //portB_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            CNPUB = CNPUB | WhichPin; //Configure pin as pull-up (set high) 
            //** NEED TO SET DIGITAL?
            returnVal = 1;
        }        
    }
    // If invalid pin/port, return false
    else{
        returnVal = 0;
    }
    return returnVal;
}

 
/****************************************************************************
 Function
    PortSetup_ConfigurePullDowns

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin(s) to be configured with weak pull-downs

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port with weak pull-downs. 
Example
   PortSetup_ConfigurePullDowns(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigurePullDowns( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    _Bool returnVal = 0;
    if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_DIO; //portA_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            CNPDA = CNPDA | WhichPin; //Configure pin as pull-down (set high)
            returnVal = 1;        
        }
    }
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_DIO; //portB_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            CNPDB = CNPDB | WhichPin; //Configure pin as pull-down (set high) 
            returnVal = 1;
        }        
    }
    // If invalid pin/port, return false
    else{
        returnVal = 0;
    }
    return returnVal;    
    

}

/****************************************************************************
 Function
    PortSetup_ConfigureOpenDrain

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin(s) to be configured as open drain outputs

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port as open drain outputs
Example
   PortSetup_ConfigureOpenDrain(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigureOpenDrain( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    _Bool returnVal = 0;
    if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_DIO; //portA_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ODCA = ODCA | WhichPin; //Configure pin as open drain (set high)
            returnVal = 1;        
        }
    }
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_DIO; //portB_DI;
        // If port and pin are allowable values, configure the pin
        if(pinCheck == WhichPin){
            ODCB = ODCB | WhichPin; //Configure pin as open drain (set high) 
            //** NEED TO SET DIGITAL?
            returnVal = 1;
        }        
    }
    // If invalid pin/port, return false
    else{
        returnVal = 0;
    }
    return returnVal;    
    

}

/****************************************************************************
 Function
    PortSetup_ConfigureChangeNotification

 Parameters
   PortSetup_Port_t: the port to be configured
   PortSetup_Pin_t: the pin(s) to be enabled for change notification

 Returns
   bool: true if port and pins represent legal ports and pins; otherwise, false

 Description
   Configures the specified pin(s) on the specified port to enable change notifications. If any bits are set in the PortSetup_Pin_t parameter, then change notifications are enabled. If that parameter is 0, then change notifications are disabled globally.
Example
   PortSetup_ConfigureChangeNotification(_Port_A, _Pin_0 | _Pin_1);
****************************************************************************/
_Bool PortSetup_ConfigureChangeNotification( PortSetup_Port_t WhichPort, PortSetup_Pin_t WhichPin){
    _Bool returnVal = 0;
    // If 0 sent, disable change notifications globally
    if(WhichPin == 0){
        CNENA = 0x0000;
        CNENB = 0x0000;
        CNCONA = BIT15LO;
        CNCONB = BIT15LO;
                
    }
    // If port and pin are allowable values, configure the pin
    else if(WhichPort == portA_val){
        pinCheck = WhichPin & portA_DIO; //portA_DI;
        if(pinCheck == WhichPin){
            CNCONA = BIT15HI;
            CNENA = CNENA | WhichPin; //Configure pin as CN (set high)
            returnVal = 1;        
        }
    }
    // If port and pin are allowable values, configure the pin
    else if(WhichPort == portB_val){
        pinCheck = WhichPin & portB_DIO; //portB_DI;
        if(pinCheck == WhichPin){
            CNCONB = BIT15HI;
            CNENB = CNENB | WhichPin; //Configure pin as CN (set high) 
            //** NEED TO SET DIGITAL?
            returnVal = 1;
        }        
    }
    // If invalid pin/port or 0 sent as pin #, return false
    else{
        returnVal = 0;
    }
    return returnVal;    
    

}